import logging
import random
import tomllib

import discord

from botbotbot.ai import AIBot
from botbotbot.shuffle import Shuffler
from botbotbot.text import TextBot
from botbotbot.tts import CambAI
from botbotbot.voice import VoiceBot
from botbotbot.wordlist import Wordlist


def main() -> None:
    description = """Discord Chaos Bot"""

    logger = logging.getLogger(__name__)
    logging.basicConfig(level=logging.INFO)

    with open("config.toml", "rb") as config_file:
        config = tomllib.load(config_file)

    guild_ids = config.get("guild_ids")
    if not (isinstance(guild_ids, list) and all(isinstance(i, int) for i in guild_ids)):
        logger.error("Guild IDs must be a list of integers.")
        guild_ids = []

    system_prompt = """Tu es une intelligence artificelle qui répond en français.
    Tu dois faire un commentaire pertinent en lien avec ce qui te sera dit.
    Ta réponse doit être très courte.
    Ta réponse doit être une seule phrase.
    TA RÉPONSE DOIT ÊTRE EN FRANÇAIS !!!"""

    aibot: AIBot | None = None

    if isinstance(key := config.get("mistral_api_key"), str):
        aibot = AIBot(
            key,
            model="mistral-large-latest",
            system_message=system_prompt,
        )
    else:
        logger.warning("No AI Bot.")

    cambai: CambAI | None = None
    if isinstance(key := config.get("cambai_api_key"), str):
        cambai = CambAI(key)
    else:
        logger.warning("No CambAI.")

    intents = discord.Intents.default()
    intents.members = True
    intents.message_content = True
    intents.reactions = True
    intents.voice_states = True

    bot = discord.Bot(description=description, intents=intents)
    shf = Shuffler()
    wl = Wordlist(bot, guild_ids)
    wl.init_events()

    text_bot = TextBot(bot, wl, aibot=aibot, shuffler=shf)
    text_bot.init_events()

    voice_bot = VoiceBot(bot, cambai, aibot=aibot, shuffler=shf, guild_ids=guild_ids)
    voice_bot.init_events()

    @bot.listen("on_ready")
    async def on_ready() -> None:
        logger.info(f"We have logged in as {bot.user}")

    @bot.slash_command(
        name="alea", guild_ids=guild_ids, description="Modifier les pseudos"
    )
    async def alea(ctx: discord.ApplicationContext) -> None:
        logger.info(f"ALEA {ctx.author}")
        await ctx.defer()
        if await shf.try_shuffle(ctx.guild):
            embed = discord.Embed(title="ALEA", color=discord.Colour.green())
            await ctx.respond(embed=embed, ephemeral=True, delete_after=30)
            logger.info("FIN ALEA")
        else:
            embed = discord.Embed(title="ERRE ALEA", color=discord.Colour.red())
            await ctx.respond(embed=embed)
            logger.info("ERRE ALEA")

    @bot.slash_command(
        name="indu", guild_ids=guild_ids, description="Poser une question à MistralAI"
    )
    async def indu(ctx: discord.ApplicationContext, prompt: str) -> None:
        if aibot is None:
            return
        logger.info(f"INDU {ctx.author} {prompt}")
        await ctx.defer()
        res_stream = await aibot.get_response_stream(prompt)

        embed = discord.Embed(
            title=prompt,
            description="",
            thumbnail="https://mistral.ai/images/favicon/favicon-32x32.png",
            color=discord.Colour.orange(),
        )
        message = await ctx.respond(embed=embed)

        async for chunk in res_stream:
            if chunk.data.choices[0].delta.content is not None:
                embed.description += chunk.data.choices[0].delta.content
                await message.edit(embed=embed)

        embed.colour = None
        await message.edit(embed=embed)
        logger.info("FIN INDU")

    @bot.slash_command(
        name="chan", guild_ids=guild_ids, description="Donner de nouveaux pseudos"
    )
    async def chan(ctx: discord.ApplicationContext, file: discord.Attachment) -> None:
        logger.info(f"CHAN {ctx.author}")
        await ctx.defer()

        members = ctx.guild.members
        members.remove(ctx.guild.owner)

        nicks = (await file.read()).decode().splitlines()
        if len(nicks) < len(members):
            embed = discord.Embed(title="ERRE CHAN", color=discord.Colour.red())
            await ctx.respond(embed=embed)
            return

        nicks = random.choices(nicks, k=len(members))
        for member, nick in zip(members, nicks):
            logger.info(member, nick)
            await member.edit(nick=nick)

        embed = discord.Embed(
            title="CHAN", description="\n".join(nicks), color=discord.Colour.green()
        )
        await ctx.respond(embed=embed)
        logger.info("FIN CHAN")

    bot.run(config.get("token"))
